﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.OleDb;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using DotNettyFrom.config;
using DotNettyFrom.model;
using DotNettyFrom.util;

namespace DotNettyFrom.db
{
    public class DbHelper
    {
        private static DbHelper Instance;
        private static readonly object locker = new object();

        private Dictionary<string, string> _idInfo { get; }

        private OleDbConnection _DbConn = null;


        private DbHelper()
        {
            _idInfo = new Dictionary<string, string>();
            _DbConn = new OleDbConnection(DBConfig.ConnStr + Application.StartupPath + @"\" + DBConfig.DbFileName);
            _DbConn.Open();
        }

        public void CloseAccessDb()
        {
            _DbConn?.Close();
        }


        public static void Init()
        {
            lock (locker)
            {
                if (Instance == null)
                {
                    Instance = new DbHelper();
                }
            }
        }

        public static DbHelper GetInstance()
        {
            if (Instance == null)
            {
                Init();
            }
            return Instance;
        }


        //reset user value
        public int ResetUserData()
        {
            string sql = "update fw set user_value=default_value";
            using (OleDbCommand inst = new OleDbCommand(sql, _DbConn))
            {
                return inst.ExecuteNonQuery();
            }
        }

        //Insert
        public int InsertCmd(WvBean wb)
        {
            if (wb == null)
                return -1;
            string sql =
                "INSERT INTO fw(my_index,MY_ID,idInfo,len,mode,desp,default_value,user_value,remark) VALUES(";
            sql += wb.index + ",";
            sql += "'" + wb.myId + "',";
            sql += "'" + wb.idInfo + "',";
            sql += "'" + wb.len + "',";
            sql += "'" + wb.mode + "',";
            sql += "'" + wb.desp + "',";
            sql += "'" + wb.defalutValue + "',";
            sql += "'" + wb.userValue + "',";
            sql += "'" + wb.remark + "')";
            using (OleDbCommand inst = new OleDbCommand(sql, _DbConn))
            {
                return inst.ExecuteNonQuery();
            }
        }

        public int InsertHistory(int _type, string his_data)
        {
            string tableName = "";
            switch (_type)
            {
                case SystemConfig.IpHisType:
                    tableName = DBConfig.IpHis;
                    break;
                case SystemConfig.AddrHisType:
                    tableName = DBConfig.AddrHis;
                    break;
                case SystemConfig.CcidHisType:
                    tableName = DBConfig.CcidHis;
                    break;
                case SystemConfig.SendHisType:
                    tableName = DBConfig.SendHis;
                    break;
                default:
                    return -1;
            }

            string sql = "insert into " + tableName + "(i_date,his_data) values('" + DateTime.Now + "','" + his_data +
                         "')";
            using (OleDbCommand inst = new OleDbCommand(sql, _DbConn))
            {
                return inst.ExecuteNonQuery();
            }
        }

        //Delete
        public int DeleteCmdById(int id)
        {
            string sql = "delete from fw where ID=" + id;
            using (OleDbCommand inst = new OleDbCommand(sql, _DbConn))
            {
                return inst.ExecuteNonQuery();
            }
        }

        public int DeleteAllFwData()
        {
            string sql = @"delete from fw";
            using (OleDbCommand inst = new OleDbCommand(sql, _DbConn))
            {
                return inst.ExecuteNonQuery();
            }
        }

        public int DeleteHistoryById(int _type, int id)
        {
            string tableName;
            switch (_type)
            {
                case SystemConfig.IpHisType:
                    tableName = DBConfig.IpHis;
                    break;
                case SystemConfig.AddrHisType:
                    tableName = DBConfig.AddrHis;
                    break;
                case SystemConfig.CcidHisType:
                    tableName = DBConfig.CcidHis;
                    break;
                case SystemConfig.SendHisType:
                    tableName = DBConfig.SendHis;
                    break;
                default:
                    return -1;
            }
            string sql = @"delete from " + tableName + " where ID=" + id;
            using (OleDbCommand inst = new OleDbCommand(sql, _DbConn))
            {
                return inst.ExecuteNonQuery();
            }
        }

        public int DeleteAllHistory(int _type)
        {
            string tableName;
            switch (_type)
            {
                case SystemConfig.IpHisType:
                    tableName = DBConfig.IpHis;
                    break;
                case SystemConfig.AddrHisType:
                    tableName = DBConfig.AddrHis;
                    break;
                case SystemConfig.CcidHisType:
                    tableName = DBConfig.CcidHis;
                    break;
                case SystemConfig.SendHisType:
                    tableName = DBConfig.SendHis;
                    break;
                default:
                    return -1;
            }
            string sql = @"delete from " + tableName;
            using (OleDbCommand inst = new OleDbCommand(sql, _DbConn))
            {
                return inst.ExecuteNonQuery();
            }
        }

        //Update
        public int UpdateCmd(WvBean wb)
        {
            string updateSQL = "update fw set my_index=" + wb.index + ",MY_ID=\"" + wb.myId + "\",idInfo=\"" +
                               wb.idInfo +
                               "\",len=\"" + wb.len +
                               "\",mode=\"" +
                               wb.mode + "\",desp=\"" + wb.desp + "\",default_value=\"" + wb.defalutValue +
                               "\",user_value=\"" +
                               wb.userValue + "\",remark=\"" + wb.remark + "\" where ID=" + wb.id;

            using (OleDbCommand inst = new OleDbCommand(updateSQL, _DbConn))
            {
                return inst.ExecuteNonQuery();
            }
        }

        //Select
        public DataTable SelectCmds(string sql = @"select * from fw")
        {
            DataTable dataTable = new DataTable();
//            string sql = @"select * from fw";

            using (OleDbCommand sqlcmd = new OleDbCommand(sql, _DbConn))
            {
                OleDbDataAdapter adapter = new OleDbDataAdapter(sqlcmd);
                dataTable.Clear();
                adapter.Fill(dataTable);

                using (OleDbDataReader reader = sqlcmd.ExecuteReader())
                {
                    while (reader.Read()) //这个read调用很重要！不写的话运行时将提示找不到数据  
                    {
                        var myId = reader.GetString(2);
                        var userValue = CheckUtil.StringIsEmpty(reader.GetString(8));
                        if (!_idInfo.ContainsKey(myId))
                            _idInfo.Add(myId, userValue);

                        /*WvBean wb = new WvBean();
                        int id = 0;
                        int index = 0;
                        var myId = "";
                        var idInfo = "";
                        var len = "";
                        var mode = "";
                        var desp = "";
                        var defalutValue = "";
                        var userValue = "";
                        var remark = "";

                        id = reader.GetInt32(0);
                        index = reader.GetInt32(1);
                        myId = reader.GetString(2);
                        idInfo = CheckUtil.StringIsEmpty(reader.GetString(3));
                        len = CheckUtil.StringIsEmpty(reader.GetString(4));
                        mode = CheckUtil.StringIsEmpty(reader.GetString(5));
                        desp = CheckUtil.StringIsEmpty(reader.GetString(6));
                        defalutValue = CheckUtil.StringIsEmpty(reader.GetString(7));
                        userValue = CheckUtil.StringIsEmpty(reader.GetString(8));
                        remark = CheckUtil.StringIsEmpty(reader.GetString(9));

                        wb.id = id;
                        wb.index = index;
                        wb.myId = myId;
                        wb.idInfo = idInfo;
                        wb.len = len;
                        wb.mode = mode;
                        wb.desp = desp;
                        wb.defalutValue = defalutValue;
                        wb.userValue = userValue;
                        wb.remark = remark;

                        if (!_idInfo.ContainsKey(myId))
                            _idInfo.Add(myId, userValue);*/
                    }
                }
            }
            return dataTable;
        }

        public WvBean SelectOneCmdById(int _id)
        {
            WvBean wb = new WvBean();
            string sql = @"select * from fw where id=" + _id;
            using (OleDbCommand sqlcmd = new OleDbCommand(sql, _DbConn))
            {
                OleDbDataAdapter adapter = new OleDbDataAdapter(sqlcmd);

                using (OleDbDataReader reader = sqlcmd.ExecuteReader())
                {
                    while (reader.Read()) //这个read调用很重要！不写的话运行时将提示找不到数据  
                    {
                        int id = 0;
                        int index = 0;
                        var myId = "";
                        var idInfo = "";
                        var len = "";
                        var mode = "";
                        var desp = "";
                        var defalutValue = "";
                        var userValue = "";
                        var remark = "";

                        id = reader.GetInt32(0);
                        index = reader.GetInt32(1);
                        myId = reader.GetString(2);
                        idInfo = CheckUtil.StringIsEmpty(reader.GetString(3));
                        len = CheckUtil.StringIsEmpty(reader.GetString(4));
                        mode = CheckUtil.StringIsEmpty(reader.GetString(5));
                        desp = CheckUtil.StringIsEmpty(reader.GetString(6));
                        defalutValue = CheckUtil.StringIsEmpty(reader.GetString(7));
                        userValue = CheckUtil.StringIsEmpty(reader.GetString(8));
                        remark = CheckUtil.StringIsEmpty(reader.GetString(9));

                        wb.id = id;
                        wb.index = index;
                        wb.myId = myId;
                        wb.idInfo = idInfo;
                        wb.len = len;
                        wb.mode = mode;
                        wb.desp = desp;
                        wb.defalutValue = defalutValue;
                        wb.userValue = userValue;
                        wb.remark = remark;

                        break;
                    }
                }
            }
            return wb;
        }

        public DataTable SelectHistory(string sql)
        {
            DataTable dataTable = new DataTable();
            using (OleDbCommand sqlcmd = new OleDbCommand(sql, _DbConn))
            {
                OleDbDataAdapter adapter = new OleDbDataAdapter(sqlcmd);
                dataTable.Clear();
                adapter.Fill(dataTable);
            }
            return dataTable;
        }
    }
}